"use client";

/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState, useRef, useEffect } from "react";
import GpLogo from "./gp-logo";
import { getEnvVariable } from "@/config";
import { useAuthentication, useAuthCache, useActiveSaveStateAuthorization, Button, DropdownItem } from "@/lib";
import { FeatureEnabled } from "@/lib/components/featureFlags";
import { Popover, PopoverTrigger, PopoverContent } from "@/lib/components/layout/Popover";
import { ManagementViews, ManagementTrigger } from "@/lib/management";

export const Navbar = () => {
  const dropdownRef = useRef<HTMLDivElement>(null);
  const auth = useAuthentication();
  const authCache = useAuthCache();
  const authorization = useActiveSaveStateAuthorization();
  const [menuOpen, setMenuOpen] = useState(false);
  const buildInfo = getEnvVariable("GRAPHPOLARIS_VERSION");
  const [managementOpen, setManagementOpen] = useState<boolean>(false);
  const [current, setCurrent] = useState<ManagementViews>("overview");
  const [sharing, setSharing] = useState<boolean>(false);

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (dropdownRef.current && !dropdownRef.current.contains(event.target as Node)) {
        setMenuOpen(false);
      }
    };
    if (menuOpen) document.addEventListener("mousedown", handleClickOutside);
    return () => {
      document.removeEventListener("mousedown", handleClickOutside);
    };
  }, [menuOpen]);

  return (
    <nav className="w-full px-2 h-12 flex flex-row items-center gap-2 md:gap-3 lg:gap-4">
      <GpLogo className="h-7" includeText={false} />
      <ManagementTrigger managementOpen={managementOpen} setManagementOpen={setManagementOpen} current={current} setCurrent={setCurrent} />

      <Button
        label="Share"
        variantType="primary"
        size="sm"
        iconComponent={sharing ? "icon-[line-md--loading-loop]" : ""}
        disabled={sharing}
        iconPosition="trailing"
        onClick={async () => {
          setSharing(true);
          await auth.shareLink();
          setSharing(false);
        }}
      />
      <div className="ml-auto">
        <div className="w-fit" ref={dropdownRef}>
          <Popover>
            <PopoverTrigger>
              <div
                className="relative inline-flex items-center justify-center w-8 h-8 overflow-hidden bg-secondary-500 rounded-full hover:bg-secondary-600 transition-colors duration-150 ease-in-out cursor-pointer"
                onClick={() => setMenuOpen(!menuOpen)}
              >
                <span className="font-medium text-light">{authCache.authentication?.username?.slice(0, 2).toUpperCase()}</span>
              </div>
            </PopoverTrigger>

            <PopoverContent className="w-56 z-30 bg-light rounded-sm border-[1px] outline-none">
              <div className="p-2 text-sm border-b">
                <h2 className="font-bold">user: {authCache.authentication?.username}</h2>
                <h3 className="text-xs break-words">session: {authCache?.sessionID}</h3>
                <h3 className="text-xs break-words">license: Creator</h3>
              </div>
              {authCache.authentication?.authenticated ? (
                <>
                  <FeatureEnabled featureFlag="SHARABLE_EXPLORATION">
                    <DropdownItem
                      value={sharing ? "Creating Share Link" : "Share"}
                      disabled={sharing}
                      onClick={async () => {
                        setSharing(true);
                        await auth.shareLink();
                        setSharing(false);
                      }}
                    />
                  </FeatureEnabled>
                  <FeatureEnabled featureFlag="VIEWER_PERMISSIONS">
                    {authCache.authorization?.savestate?.W && authorization.database?.W && (
                      <DropdownItem
                        value="Viewer Permissions"
                        onClick={() => {
                          setManagementOpen(true);
                          setCurrent("members");
                        }}
                      />
                    )}
                  </FeatureEnabled>
                  <DropdownItem value="Settings" onClick={() => {}} />
                  <DropdownItem
                    value="Log out"
                    onClick={() => {
                      const current = encodeURIComponent(window.location.href);
                      location.replace(`${getEnvVariable("GP_AUTH_URL")}if/flow/default-invalidation-flow/?next=${current}`);
                    }}
                  />
                  {authCache.authorization?.demoUser?.R && (
                    <DropdownItem
                      value="Impersonate Demo User"
                      onClick={() => {
                        const url = new URL(window.location.href);
                        url.searchParams.append("impersonateID", "demoUser");
                        location.replace(url.toString());
                      }}
                    />
                  )}
                </>
              ) : (
                <>
                  <DropdownItem value="Login" onClick={() => {}} />
                </>
              )}

              {authCache.authentication?.roomID && (
                <div className="p-2 border-b">
                  <h3 className="text-xs break-words">Share ID: {authCache.authentication?.roomID}</h3>
                </div>
              )}
              <div className="p-2 border-t">
                <h3 className="text-xs">Version: {buildInfo}</h3>
              </div>
            </PopoverContent>
          </Popover>
        </div>
      </div>
    </nav>
  );
};
