import { BoolFilters } from './boolFilters';
import {
  GeneralDescription,
  NumberFunctionTypes,
  NumberFilterTypes,
  StringFilterTypes,
  StringFunctionTypes,
  NumberAggregationTypes,
  BoolFilterTypes,
} from './general';
import { MathAggregations } from './numberAggregations';
import { MathFilters } from './numberFilters';
import { NumberFunctions } from './numberFunctions';
import { StringFilters } from './stringFilters';
import { StringFunctions } from './stringFunctions';

export type AllLogicTypes = NumberFilterTypes | NumberFunctionTypes | BoolFilterTypes | NumberAggregationTypes | StringFilterTypes | StringFunctionTypes;
export type AllLogicDescriptions = GeneralDescription<AllLogicTypes>;

export const AllLogicMap: Record<string, AllLogicDescriptions> = {
  ...Object.fromEntries(Object.values(MathFilters).map((x) => [x.key, x])),
  ...Object.fromEntries(Object.values(BoolFilters).map((x) => [x.key, x])),
  ...Object.fromEntries(Object.values(NumberFunctions).map((x) => [x.key, x])),
  ...Object.fromEntries(Object.values(MathAggregations).map((x) => [x.key, x])),
  ...Object.fromEntries(Object.values(StringFilters).map((x) => [x.key, x])),
  ...Object.fromEntries(Object.values(StringFunctions).map((x) => [x.key, x])),
};

export * from './numberFunctions';
export * from './numberFilters';
export * from './stringFunctions';
export * from './stringFilters';
